/**
 * (c) Universidad de Oviedo.
 * 
 * This is part of the "SQLTools" set of tools for database testing.
 * See SQLTools terms of use at: http://in2test.lsi.uniovi.es/sqltools/common/license.aspx
 * 
 * Sample program for testing the SQLMutation Web Service.
 * 
 * @author Javier Tuya
 * @date August 2006
 * Modified August 2007: Add sample code to extract the mutated SQL queries from the xml response
 * 
 * THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. 
 * EXCEPT WHEN OTHERWISE STATED IN WRITING, THE COPYRIGHT HOLDER PROVIDE THE PROGRAM "AS IS" 
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE 
 * QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, 
 * YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
 * 
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING, ANY COPYRIGHT 
 * HOLDER WILL BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL 
 * OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM 
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES 
 * SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER 
 * PROGRAMS), EVEN IF SUCH HOLDER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * 
 */
package in2test.samples.sqlmutationws.java;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.StringBufferInputStream;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextArea;
import javax.swing.JScrollPane;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import es.uniovi.lsi.in2test.sqlmutationws.SQLMutationWSSoapProxy;

public class Client extends JPanel implements ActionListener {

	//User interface objects and methods

	static final long serialVersionUID=0;

	public  JFrame mainFrame;
	private JLabel lblSql=new JLabel("Query:     ");
	private JLabel lblSchema=new JLabel("Schema: ");
	private JLabel lblMutants=new JLabel("Mutants (XML): ");
	private JLabel lblMutatedQueries=new JLabel("Mutants (SQL): ");
	private JLabel status=new JLabel("");
	private JTextArea sql=new JTextArea(5,60);
	private JTextArea schema=new JTextArea(7,60);
	private JTextArea mutants=new JTextArea(12,60);
	private JTextArea mutatedQueries=new JTextArea(8,60);
	private JButton generate=new JButton("Get Mutants");

	public Client(String nombre) {
		mainFrame = new JFrame(nombre);
		//layout
		this.setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));

		Box bxSql=Box.createHorizontalBox();
		bxSql.add(lblSql);
		JScrollPane spSql=new JScrollPane(sql);
		bxSql.add(spSql);
		this.add(bxSql);

		Box bxSchema=Box.createHorizontalBox();
		bxSchema.add(lblSchema);
		JScrollPane spSchema=new JScrollPane(schema);
		bxSchema.add(spSchema);
		this.add(bxSchema);

		Box bxGenerate=Box.createHorizontalBox();
		bxGenerate.add(generate);
		bxGenerate.add(status);
		this.add(bxGenerate);

		Box bxMutants=Box.createHorizontalBox();
		bxMutants.add(lblMutants);
		JScrollPane spMutants=new JScrollPane(mutants);
		bxMutants.add(spMutants);
		this.add(bxMutants);

		Box bxMutatedQueries=Box.createHorizontalBox();
		bxMutatedQueries.add(lblMutatedQueries);
		JScrollPane spMutatedQueries=new JScrollPane(mutatedQueries);
		bxMutatedQueries.add(spMutatedQueries);
		this.add(bxMutatedQueries);

		//install event listener
		generate.setName("generate");
		generate.setActionCommand("generate");
		generate.addActionListener(this);
		// pack and show
		mainFrame.setContentPane(this);
		mainFrame.pack();
		mainFrame.show();//(true);
	}

	public void actionPerformed(ActionEvent e) {
		if ("generate".equals(e.getActionCommand())) {
			System.out.println("Calling Service");
			callService();
		}
	}
	protected void msgBox(String msg) {
		JOptionPane.showMessageDialog(mainFrame,msg,
				"Exception Message",JOptionPane.ERROR_MESSAGE);
	}

	public static void main(String[] args) {
		new Client("SQLMutationWSClient");
	}

	/**
	 * Main method. Gets the sql and schema from the UI, calls the service and presents results
	 */
	private void callService() {
		try {
			mutants.setText("");
			status.setText("Calling the Web Service ...");
			mainFrame.update(mainFrame.getGraphics());
			// Creates the proxy object and calls the service
			SQLMutationWSSoapProxy proxy=new SQLMutationWSSoapProxy();
			String result=proxy.getMutants(sql.getText(), schema.getText(), "");
			mutants.setText(result);
			mutatedQueries.setText(xml2sql(result));
			status.setText("Success");
		} catch (Exception e) {
			e.printStackTrace();
			status.setText("Exception: " + e.getMessage());
			msgBox("Exception: " + e.toString());
		}
	}

	/**
	 * Extract the mutated sql queries from the xml
	 * @param xmlResult xml string returned from the service
	 * @return string with all mmutated queries
	 */
	private String xml2sql(String xmlResult) {
		String sqlResult="";
		//System.out.println("Begin");
		//assume that the xml has been saved in a file in the temp directory
		//First load into a Document
		Document doc=newDocument(xmlResult);
		//get root node: sqlmutationws:
		Node n=doc.getFirstChild();

		//iterate over all components to discover version, error, sql and mutants nodes
		NodeList componentList=n.getChildNodes();
		for (int c=0; c<componentList.getLength(); c++) {
			Node component=componentList.item(c);
			//System.out.println(component.getNodeName());
			//process only the mutants node
			//(checking for errors would also be important)
			if (component.getNodeName().equals("mutants")) {
				//iterate each mutant
				NodeList mutantList=component.getChildNodes();
				for (int i=0; i<mutantList.getLength(); i++) {
					//current mutant
					Node mutant=mutantList.item(i);
					//System.out.println(mutant.getNodeName());
					//skip id, category, etc. until sql tag found
					NodeList itemList=mutant.getChildNodes();
					for (int j=0; j<itemList.getLength(); j++) {
						if (itemList.item(j).getNodeName().equals("sql")) {
							//sql node found, get the inner text (first child, it must be a text node)
							String sql=itemList.item(j).getFirstChild().getTextContent();
							System.out.println("SQL of mutant: " + sql);
							sqlResult+=sql+"\n";
						}
					}
				}

			}
		}
		return sqlResult;
	}

	/**
	 * Constructs an xml Document from an xml string
	 * @param s string with the xml
	 * @return xml document returned
	 */
	public Document newDocument(String s){
		Document doc=null;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		//factory.setValidating(true);   
		//factory.setNamespaceAware(true);
		try {
			DocumentBuilder builder = factory.newDocumentBuilder();
			//doc = builder.parse(new java.io.File(s) ); //if s is a file name
			doc = builder.parse(new StringBufferInputStream(s) ); //if s is a string containing the xml

		} catch (SAXParseException spe) {
			throw new RuntimeException("XDocument.loadXml",spe);
			///CLOVER:OFF
		} catch (SAXException sxe) {
			// Error generated during parsing)
			Exception  x = sxe;
			if (sxe.getException() != null)
				x = sxe.getException();
			x.printStackTrace();
			throw new RuntimeException("XDocument.loadXml",sxe);

		} catch (ParserConfigurationException pce) {
			// Parser with specified options can't be built
			pce.printStackTrace();
			throw new RuntimeException("XDocument.loadXml",pce);
		} catch (IOException ioe) {
			// I/O error
			ioe.printStackTrace();
			throw new RuntimeException("XDocument.loadXml",ioe);
			///CLOVER:ON
		}	
		return doc;
	}

}
